#pragma once
#include <string>
#include "PacketTypeSystem.h"

/// @name Packet Types Primitives
/// @relates WMPacket
//@{
namespace WorldMachine {
namespace Packet {
	const TypeHandle ANY		= 0;
	const TypeHandle parameter	= 1;			// Parameter Packet
	const TypeHandle scalar		= 2;			// A 0..1 scalar type
	const TypeHandle hfield		= 4;			// Heightfields
	const TypeHandle rgb		= 8;			// RGB Bitmaps
	const TypeHandle volumetric	= 16;			// Volume Field (not available currently)
	const TypeHandle transform	= 32;			// A packet encoding a 2D affine transformation matrix (3x3)
	const TypeHandle text		= 64;			// Text
	const TypeHandle combo		= 128;			// Combined-terrain Datatype
	const TypeHandle mesh		= 256;			// Mesh
	const TypeHandle nulltype	= 2048;			// Set to this type when it should not match any other type
	const TypeHandle curve		= 8192;			// Curve object type
}
}
const WorldMachine::TypeHandle WMP_areabase	= 512;			// WMAreaPacket-derived class
const WorldMachine::TypeHandle WMP_gridbase	= 1024;			// WMGridPacket-derived class
const WorldMachine::TypeHandle WMP_proxytype = 4096;			// WMAreaPacket proxy, used for sizing grid packet types
const WorldMachine::TypeHandle WMP_dyntype	= 1 << 14;		// Dynamic type packet
const WorldMachine::TypeHandle WMP_reserved1 = 1 << 15;		// Future datatypes...
const WorldMachine::TypeHandle WMP_usertype_base = 1 << 20;	// Base index for user-defined types; usertype_base-1 is the highest type handle available to non-registered types!
const WorldMachine::PacketTypeMany WMP_PARAMS	= WorldMachine::PacketType(WorldMachine::Packet::parameter) | WorldMachine::PacketType(WorldMachine::Packet::scalar);
// If you want to create new, user-defined packet types, CONTACT ME FIRST.
// I will give you the bitcode to use to prevent collisions with any other packet type

/// Return a string with type(s) of packet by ID
std::string getPacketTypeString(const WorldMachine::PacketType type, const std::string concat_str = ", ");
//@}