#pragma once
#include "ForwardDef.h"
#include "parampacket.h"
#include "CurveTypes.h"

namespace WorldMachine {



/// Transfers a curve datatype through the device network.
/// The curve datatype is unique in that it is actually a function object.
/// The curve packet holds a Curve object, which can then be queried against to evaluate the curve at arbitrary points
/// @ingroup data
class CurvePacket :
	public WMPacket
{
public:
	static CurvePointer CreateNew(Curve1DInterface *curve_create);
	CurvePacket &operator=(const CurvePacket &other);

	virtual size_t getHeapUsage() { return 0;	};

	virtual WorldMachine::PacketType getType() const { return WorldMachine::Packet::curve; };
	static WorldMachine::PacketType getStaticType() { return WorldMachine::Packet::curve; };

	virtual const char *getTypeName() const { return "Curve"; };
	virtual void Clear() { };
	virtual PacketPointer Clone() const;

	virtual bool Load(std::istream &input);
	virtual bool Save(std::ostream &output);	

	virtual dword Color() { return 0x03fffff; };

	void setCurve(Curve1DInterface *newcurve) { if (curve) curve->Release(); curve = newcurve; };
	Curve1DInterface *getCurve() const { return curve; };

protected:
	CurvePacket(void);
	~CurvePacket(void);
	friend class WMPacketFactory;
	virtual void resetContents() { };

	virtual bool queryType( WorldMachine::PacketType const&type ) const;

	virtual bool isTypePageable() { return false; };

	Curve1DInterface *curve;
};


/** Accessor inline function to provide a safer way to cast WMPacket to ParamPacket
// in debug mode it will assert if the packet is not a ParamPacket; in release you pay nothing
// framework should never allow this anyways, but it never hurts to be careful!
	@relates ParamPacket
*/
inline CurvePointer CURVE(PacketPointer pack) {
	return CASTPACKET<WorldMachine::CurvePacket>(pack);
};

}
